@@ -16,10 +16,9 @@ import types
 
 from collections import MutableMapping, MutableSequence, MutableSet, namedtuple
 from . import memory
-from .utils import (first_is_not, isclassattr, isscalar, ndarraywrap,
-                    strenum, strshape, tointtuple)
-from .decorators import (real, idempotent, involutary, square, symmetric,
-                         inplace)
+from .utils import (assignment_operation, first_is_not, isclassattr, isscalar,
+                    ndarraywrap, strenum, strshape, tointtuple)
+from .decorators import linear, real, idempotent, involutary, symmetric, inplace
 
 __all__ = [
     'Operator',
@@ -30,17 +29,19 @@ __all__ = [
     'BlockRowOperator',
     'BroadcastingOperator',
     'CompositionOperator',
+    'ConstantOperator',
     'DiagonalOperator',
+    'HomothetyOperator',
     'IdentityOperator',
     'MultiplicationOperator',
     'ReshapeOperator',
-    'ScalarOperator',
     'ZeroOperator',
     'asoperator',
     'I',
     'O',
 ]
 
+
 class OperatorFlags(namedtuple('OperatorFlags',
                                ['linear',
                                 'square',     # shapein == shapeout
@@ -64,6 +65,7 @@ class OperatorFlags(namedtuple('OperatorFlags',
         return self.__class__.__name__ + '(\n  ' + ',\n  '.join([f + str(v) \
             for f,v in zip(fields,self)]) + ')'
 
+
 class OperatorRule(object):
     """ Binary rule on operators.
 
@@ -741,7 +743,7 @@ class Operator(object):
                 op.flags = op.flags._replace(linear=True, square=True)
             if 'orthogonal' in flags:
                 op.flags = op.flags._replace(real=True)
-            if 'idempotent' in flags or 'involutary' in flags:
+            if 'involutary' in flags:
                 op.flags = op.flags._replace(square=True)
         elif flags is not None:
             raise TypeError("Invalid input flags: '{0}'.".format(flags))
@@ -898,19 +900,21 @@ class Operator(object):
         input = np.array(input, copy=False, subok=True)
 
         shapeout = self.reshapein(input.shape)
-        if shapeout is None:
-            shapeout = input.shape
         dtype = self._find_common_type([input.dtype, self.dtype])
         input = np.array(input, dtype=dtype, subok=False, copy=False)
         if output is not None:
+            if not isinstance(output, np.ndarray):
+                raise TypeError('The output argument is not an ndarray.')
             if output.dtype != dtype:
                 raise ValueError("The output has an invalid dtype '{0}'. Expect"
                     "ed dtype is '{1}'.".format(output.dtype, dtype))
-            if output.shape != shapeout:
+            if shapeout is not None and shapeout != output.shape:
                 raise ValueError("The output has an invalid shape '{0}'. Expect"
                     "ed shape is '{1}'.".format(output.shape, shapeout))
             output = output.view(np.ndarray)
         else:
+            if shapeout is None:
+                shapeout = input.shape
             output = memory.allocate(shapeout, dtype, None, self.__name__)[0]
         return input, output
 
@@ -947,7 +951,7 @@ class Operator(object):
         return AdditionOperator([self, -other])
 
     def __neg__(self):
-        return ScalarOperator(-1) * self
+        return HomothetyOperator(-1) * self
 
     def __str__(self):
         if self.shapein is not None:
@@ -996,6 +1000,7 @@ class Operator(object):
                 a += [var + '=' + s]
         return self.__name__ + '(' + ', '.join(a) + ')'
 
+
 def asoperator(operator, shapein=None, shapeout=None):
     if isinstance(operator, Operator):
         if shapein and operator.shapein and shapein != operator.shapein:
@@ -1026,7 +1031,7 @@ def asoperator(operator, shapein=None, shapeout=None):
                         flags='linear')
     
     if isscalar(operator):
-        return ScalarOperator(operator)
+        return HomothetyOperator(operator)
 
     return asoperator(scipy.sparse.linalg.aslinearoperator(operator))
 
@@ -1109,6 +1114,7 @@ class CommutativeCompositeOperator(CompositeOperator):
     Class for commutative operator, such as addition or multiplication
     """
     def __new__(cls, operands, operation, *args, **keywords):
+        operands = cls._validate_constants(operands)
         return CompositeOperator.__new__(cls, operands, *args, **keywords)
 
     def __init__(self, operands, operation, *args, **keywords):
@@ -1210,7 +1216,7 @@ class CommutativeCompositeOperator(CompositeOperator):
             i += 1
 
         # move this up to avoid creations of temporaries
-        i = [i for i,o in enumerate(ops) if isinstance(o, ScalarOperator)]
+        i = [i for i,o in enumerate(ops) if isinstance(o, HomothetyOperator)]
         if len(i) > 0:
             ops.insert(0, ops[i[0]])
             del ops[i[0]+1]
@@ -1218,6 +1224,14 @@ class CommutativeCompositeOperator(CompositeOperator):
                 del ops[0]
         return ops
 
+    @classmethod
+    def _validate_constants(cls, operands):
+        for i, op in enumerate(operands):
+            if isinstance(op, (int, float, complex, np.bool_, np.number,
+                          np.ndarray)):
+                operands[i] = ConstantOperator(op)
+        return operands
+
 
 class AdditionOperator(CommutativeCompositeOperator):
     """
@@ -1996,12 +2010,19 @@ class BlockDiagonalOperator(BlockOperator):
 
     def direct(self, input, output):
         if None in self.partitionout:
-            shapeins = self._get_shapes(input.shape, self.partitionin,
-                                        self.axisin, self.new_axisin)
-            partitionout = [op.reshapein(s)[self.axisout] \
-                            for op,s in zip(self.operands, shapeins)]
+            partitionout = list(self.partitionout)
+            for i, op in enumerate(self.operands):
+                if partitionout[i] is not None:
+                    continue
+                if self.partitionin[i] is None:
+                    raise ValueError('The shape of an operator with implicit pa'
+                        'rtition cannot be inferred.')
+                shapein = list(input.shape)
+                shapein[self.axisin] = self.partitionin[i]
+                partitionout[i] = op.reshapein(shapein)[self.axisout]
         else:
             partitionout = self.partitionout
+
         destin = 0
         destout = 0
         for op, nin, nout in zip(self.operands, self.partitionin, partitionout):
@@ -2068,9 +2089,16 @@ class BlockColumnOperator(BlockOperator):
         
     def direct(self, input, output):
         if None in self.partitionout:
-            raise NotImplementedError()
+            partitionout = list(self.partitionout)
+            for i, op in enumerate(self.operands):
+                if partitionout[i] is not None:
+                    continue
+                partitionout[i] = op.reshapein(input.shape)[self.axisout]
+        else:
+            partitionout = self.partitionout
+
         dest = 0
-        for op, n in zip(self.operands, self.partitionout):
+        for op, n in zip(self.operands, partitionout):
             if self.new_axisout is not None:
                 self.sliceout[self.new_axisout] = dest
             else:
@@ -2132,10 +2160,16 @@ class BlockRowOperator(BlockOperator):
 
     def direct(self, input, output):
         if None in self.partitionin:
-            raise NotImplementedError()
+            partitionin = list(self.partitionin)
+            for i, op in enumerate(self.operands):
+                if partitionin[i] is None:
+                    partitionin[i] = input.shape[self.axisin]
+        else:
+            partitionin = self.partitionin
+
         work = np.zeros_like(output)
         dest = 0
-        for op, n in zip(self.operands, self.partitionin):
+        for op, n in zip(self.operands, partitionin):
             if self.new_axisin is not None:
                 self.slicein[self.new_axisin] = dest
             else:
@@ -2152,6 +2186,7 @@ class BlockRowOperator(BlockOperator):
         return '[[ ' + ' '.join(operands) + ' ]]'
 
 
+@linear
 @real
 @inplace
 class ReshapeOperator(Operator):
@@ -2169,6 +2204,8 @@ class ReshapeOperator(Operator):
             raise ValueError('The input shape is None.')
         if shapeout is None:
             raise ValueError('The output shape is None.')
+        shapein = tointtuple(shapein)
+        shapeout = tointtuple(shapeout)
         if np.product(shapein) != np.product(shapeout):
             raise ValueError('The total size of the output must be unchanged.')
         if shapein == shapeout:
@@ -2190,14 +2227,13 @@ class ReshapeOperator(Operator):
         return strshape(self.shapeout) + '←' + strshape(self.shapein)
 
 
-@square
 class BroadcastingOperator(Operator):
     """
     Abstract class for operators that operate on a data array and
     the input array, and for which broadcasting of the data array across
     the input array is required.
     """
-    def __init__(self, data, broadcast='disabled', shapein=None, dtype=None,
+    def __init__(self, data, broadcast='disabled', shapeout=None, dtype=None,
                  **keywords):
         if data is None:
             raise ValueError('The input data is None.')
@@ -2205,19 +2241,133 @@ class BroadcastingOperator(Operator):
         if dtype is None:
             dtype = data.dtype
         data = np.array(data, dtype, order='c', copy=False)
+        if data.ndim == 0:
+            broadcast = 'scalar'
         broadcast = broadcast.lower()
         values = ('fast', 'slow', 'disabled', 'scalar')
         if broadcast not in values:
             raise ValueError("Invalid value '{0}' for the broadcast keyword. Ex"
                 "pected values are {1}.".format(broadcast, strenum(values)))
         if broadcast == 'disabled':
-            if shapein not in (None, data.shape):
+            if shapeout not in (None, data.shape):
                 raise ValueError("The input shapein is incompatible with the da"
                                  "ta shape.")
-            shapein = data.shape
+            shapeout = data.shape
         self.broadcast = broadcast
         self.data = data
-        Operator.__init__(self, shapein=shapein, dtype=dtype, **keywords)
+        Operator.__init__(self, shapeout=shapeout, dtype=dtype, **keywords)
+        self.add_rule('{BroadcastingOperator}.', lambda o: self._rule_broadcast(
+                      o, np.add), AdditionOperator)
+        self.add_rule('{BroadcastingOperator}.', lambda o: self._rule_broadcast(
+                      o, np.multiply))
+
+    def _rule_broadcast(self, d, operation):
+        # check the direct subclasses of Broadcasting for each operand
+        i1 = self.__class__.__mro__.index(BroadcastingOperator) - 1
+        try:
+            i2 = d.__class__.__mro__.index(BroadcastingOperator) - 1
+        except ValueError:
+            i2 = -1
+        if i1 == i2 == -1:
+            cls = BroadcastingOperator
+        elif i1 == -1:
+            cls = d.__class__.__mro__[i2]
+        elif i2 == -1:
+            cls = self.__class__.__mro__[i1]
+        else:
+            cls = self.__class__.__mro__[i1]
+            if cls is not d.__class__.__mro__[i2]:
+                return None
+
+        # check broadcast
+        b = set([self.broadcast, d.broadcast])
+        if 'slow' in b and 'fast' in b:
+            return None
+        if 'disabled' in b:
+            broadcast = 'disabled'
+        elif 'slow' in b:
+            broadcast = 'slow'
+        elif 'fast' in b:
+            broadcast = 'fast'
+        else:
+            broadcast = 'scalar'
+        if 'fast' in b:
+            data = operation(self.data.T, d.data.T).T
+        else:
+            data = operation(self.data, d.data)
+        
+        shapeout = self.shapeout if self.shapeout is not None else d.shapeout
+        return cls(data, broadcast, shapeout=shapeout, dtype=\
+                   self._find_common_type([self.dtype, d.dtype]))
+
+
+@symmetric
+@inplace
+class DiagonalOperator(BroadcastingOperator):
+    """
+    Diagonal operator.
+
+    Arguments
+    ---------
+
+    data : ndarray
+      The diagonal coefficients
+
+    broadcast : 'fast' or 'disabled' (default 'disabled')
+      If broadcast == 'fast', the diagonal is broadcasted along the fast
+      axis.
+
+    Exemple
+    -------
+    >>> A = DiagonalOperator(arange(1, 6, 2))
+    >>> A.todense()
+
+    array([[1, 0, 0],
+           [0, 3, 0],
+           [0, 0, 5]])
+
+    >>> A = DiagonalOperator(arange(1, 3), broadcast='fast', shapein=(2, 2))
+    >>> A.todense()
+
+    array([[1, 0, 0, 0],
+           [0, 1, 0, 0],
+           [0, 0, 2, 0],
+           [0, 0, 0, 2]])
+    """
+    def __init__(self, data, broadcast='disabled', **keywords):
+        data = np.asarray(data)
+        if not isinstance(self, HomothetyOperator) and \
+           np.all(data == data.flat[0]):
+            if broadcast == 'disabled' and data.ndim > 0:
+                keywords['shapein'] = data.shape
+            self.__class__ = HomothetyOperator
+            self.__init__(data.flat[0], **keywords)
+            return
+        BroadcastingOperator.__init__(self, data, broadcast, **keywords)
+
+    def direct(self, input, output):
+        if self.broadcast == 'fast':
+            np.multiply(input.T, self.data.T, output.T)
+        else:
+            np.multiply(input, self.data, output)
+
+    def conjugate_(self, input, output):
+        if self.broadcast == 'fast':
+            np.multiply(input.T, np.conjugate(self.data).T, output.T)
+        else:
+            np.multiply(input, np.conjugate(self.data), output)
+
+    def inverse(self, input, output):
+        if self.broadcast == 'fast':
+            np.divide(input.T, self.data.T, output.T)
+        else:
+            np.divide(input, self.data, output)
+
+    def inverse_conjugate(self, input, output):
+        if self.broadcast == 'fast':
+            np.divide(input.T, np.conjugate(self.data).T, output.T)
+        else:
+            np.divide(input, np.conjugate(self.data), output)
 
     def reshapein(self, shape):
         if shape is None or self.data.size == 1:
@@ -2262,107 +2412,16 @@ class BroadcastingOperator(Operator):
         return v
 
 
-@symmetric
 @inplace
-class DiagonalOperator(BroadcastingOperator):
-    """
-    Diagonal operator.
-
-    Arguments
-    ---------
-
-    data : ndarray
-      The diagonal coefficients
-
-    broadcast : 'fast' or 'disabled' (default 'disabled')
-      If broadcast == 'fast', the diagonal is broadcasted along the fast
-      axis.
-
-    Exemple
-    -------
-    >>> A = DiagonalOperator(arange(1, 6, 2))
-    >>> A.todense()
-
-    array([[1, 0, 0],
-           [0, 3, 0],
-           [0, 0, 5]])
-
-    >>> A = DiagonalOperator(arange(1, 3), broadcast='fast', shapein=(2, 2))
-    >>> A.todense()
-
-    array([[1, 0, 0, 0],
-           [0, 1, 0, 0],
-           [0, 0, 2, 0],
-           [0, 0, 0, 2]])
-    """
-    def __init__(self, data, broadcast='disabled', **keywords):
-        data = np.asarray(data)
-        if not isinstance(self, ScalarOperator) and \
-           np.all(data == data.flat[0]):
-            self.__class__ = ScalarOperator
-            self.__init__(data.flat[0], **keywords)
-            return
-        BroadcastingOperator.__init__(self, data, broadcast, **keywords)
-        self.add_rule('{DiagonalOperator}.', lambda o: self._rule_diagonal(o,
-                      np.add), AdditionOperator)
-        self.add_rule('{DiagonalOperator}.', lambda o: self._rule_diagonal(o,
-                      np.multiply))
-
-    def direct(self, input, output):
-        if self.broadcast == 'fast':
-            np.multiply(input.T, self.data.T, output.T)
-        else:
-            np.multiply(input, self.data, output)
-
-    def conjugate_(self, input, output):
-        if self.broadcast == 'fast':
-            np.multiply(input.T, np.conjugate(self.data).T, output.T)
-        else:
-            np.multiply(input, np.conjugate(self.data), output)
-
-    def inverse(self, input, output):
-        if self.broadcast == 'fast':
-            np.divide(input.T, self.data.T, output.T)
-        else:
-            np.divide(input, self.data, output)
-
-    def inverse_conjugate(self, input, output):
-        if self.broadcast == 'fast':
-            np.divide(input.T, np.conjugate(self.data).T, output.T)
-        else:
-            np.divide(input, np.conjugate(self.data), output)
-
-    def _rule_diagonal(self, d, operation):
-        b = set([self.broadcast, d.broadcast])
-        if 'slow' in b and 'fast' in b:
-            return None
-        if 'disabled' in b:
-            broadcast = 'disabled'
-        elif 'slow' in b:
-            broadcast = 'slow'
-        elif 'fast' in b:
-            broadcast = 'fast'
-        else:
-            broadcast = 'scalar'
-        if 'fast' in b:
-            data = operation(self.data.T, d.data.T).T
-        else:
-            data = operation(self.data, d.data)
-        shapein = self.shapein if self.shapein is not None else d.shapein
-        return DiagonalOperator(data, broadcast, shapein=shapein, dtype= \
-            self._find_common_type([self.dtype, d.dtype]))
-
-
-@inplace
-class ScalarOperator(DiagonalOperator):
+class HomothetyOperator(DiagonalOperator):
     """
     Multiplication by a scalar.
 
     """
     def __init__(self, data, **keywords):
         data = np.asarray(data)
-        if data.size != 1:
-            raise ValueError("Invalid data size '{0}' for ScalarOperator." \
+        if data.ndim > 0:
+            raise ValueError("Invalid data size '{0}' for HomothetyOperator." \
                              .format(data.size))
         if not isinstance(self, ZeroOperator) and data == 0:
             self.__class__ = ZeroOperator
@@ -2380,13 +2439,12 @@ class ScalarOperator(DiagonalOperator):
 
     def associated_operators(self):
         return {
-            'C' : ScalarOperator(np.conjugate(self.data), shapein=self.shapein,
-                                 dtype=self.dtype),
-            'I' : ScalarOperator(1/self.data if self.data != 0 else np.nan,
-                                 shapein=self.shapein, dtype=self.dtype),
-            'IC' : ScalarOperator(np.conjugate(1/self.data) if self.data != 0 \
-                                  else np.nan, shapein=self.shapein,
-                                  dtype=self.dtype)
+            'C' : HomothetyOperator(np.conjugate(self.data),
+                      shapein=self.shapein, dtype=self.dtype),
+            'I' : HomothetyOperator(1/self.data if self.data != 0 else np.nan,
+                      shapein=self.shapein, dtype=self.dtype),
+            'IC' :HomothetyOperator(np.conjugate(1/self.data) if self.data != 0\
+                      else np.nan, shapein=self.shapein, dtype=self.dtype)
         }
 
     def __str__(self):
@@ -2400,10 +2458,10 @@ class ScalarOperator(DiagonalOperator):
             return None
         if self.shapein is None or operator.shapein is not None:
             #XXX this case should not happen
-            if isinstance(operator, ScalarOperator):
+            if isinstance(operator, HomothetyOperator):
                 return None
             return (self, operator)
-        return (ScalarOperator(self.data, dtype=self.dtype), operator,
+        return (HomothetyOperator(self.data, dtype=self.dtype), operator,
                 IdentityOperator(self.shapein))
 
 
@@ -2411,9 +2469,9 @@ class ScalarOperator(DiagonalOperator):
 @idempotent
 @involutary
 @inplace
-class IdentityOperator(ScalarOperator):
+class IdentityOperator(HomothetyOperator):
     """
-    A subclass of ScalarOperator with data = 1.
+    A subclass of HomothetyOperator with data = 1.
 
     Exemple
     -------
@@ -2431,7 +2489,7 @@ class IdentityOperator(ScalarOperator):
 
     """
     def __init__(self, shapein=None, **keywords):
-        ScalarOperator.__init__(self, 1, shapein=shapein, **keywords)
+        HomothetyOperator.__init__(self, 1, shapein=shapein, **keywords)
         self.add_rule('.{Operator}', self._rule_identity)
 
     def direct(self, input, output):
@@ -2450,22 +2508,80 @@ class IdentityOperator(ScalarOperator):
             return operator
 
 
-@real
+@idempotent
 @inplace
-class ZeroOperator(ScalarOperator):
+class ConstantOperator(BroadcastingOperator):
     """
-    A subclass of ScalarOperator with data = 0.
+    Non-linear constant operator.
     """
-    def __init__(self, shapein=None, shapeout=None, dtype=None, reshapein=None,
-                 reshapeout=None, **keywords):
-        if shapein is not None and shapein == shapeout or reshapein is None and\
-           reshapeout is None:
-            flags = 'square, symmetric, idempotent'
+    def __init__(self, data, broadcast='disabled', **keywords):
+        data = np.asarray(data)
+        if data.ndim > 0 and np.all(data == data.flat[0]):
+            if broadcast == 'disabled':
+                keywords['shapeout'] = data.shape
+            self.__init__(data.flat[0], 'scalar', **keywords)
+            return
+        if not isinstance(self, ZeroOperator) and data.ndim == 0  and data == 0:
+            self.__class__ = ZeroOperator
+            self.__init__(**keywords)
+            return
+        BroadcastingOperator.__init__(self, data, broadcast, **keywords)
+        #self.add_rule('.{Operator}', self._rule_constant_comp)
+        #self.add_rule('{Operator}.', self._rule_comp_constant)
+
+    def __neg__(self):
+        return ConstantOperator(-self.data, broadcast=self.broadcast,
+            shapein=self.shapein, shapeout=self.shapeout,
+            reshapein=self._reshapein, reshapeout=self._reshapeout,
+            dtype=self.dtype)
+
+    def associated_operators(self):
+        return {
+            'C' : ConstantOperator(self.data.conjugate(),
+                      broadcast=self.broadcast, shapein=self.shapein,
+                      shapeout=self.shapeout, reshapein=self._reshapein,
+                      reshapeout=self._reshapeout, dtype=self.dtype)
+        }
+
+    def direct(self, input, output, operation=assignment_operation):
+        if self.broadcast == 'fast':
+            operation(output.T, self.data.T)
         else:
-            flags = None
-        ScalarOperator.__init__(self, 0, shapein=shapein, shapeout=shapeout,
-                                  reshapein=reshapein, reshapeout=reshapeout,
-                                  dtype=dtype, flags=flags, **keywords)
+            operation(output, self.data)
+
+    @staticmethod
+    def _combine_operators(self, o1, o2):
+        result = ConstantOperator(shapein=o2.shapein or o2.reshapeout(o1.shapein),
+            shapeout=o1.shapeout or o2.shapeout,
+            reshapein=lambda s: o1.reshapein(o2.reshapein(s)),
+            reshapeout=lambda s: o2.reshapeout(o1.reshapeout(s)),
+            dtype=self._find_common_type([o1.dtype, o2.dtype]))
+        result.toshapein = lambda v: o2.toshapein(v)
+        return result
+
+    def _rule_constant_comp(self, op):
+        return self._combine_operators(self, op)
+
+    def _rule_comp_constant(self, op):
+        if self.shapein is not None:
+            shape = self.shapein
+        if not op.flags.linear:
+            return None
+        return self._combine_operators(op, self)
+
+    def __str__(self):
+        return str(self.data)
+
+
+@linear
+@real
+class ZeroOperator(ConstantOperator):
+    """
+    A subclass of ConstantOperator with data = 0.
+    """
+    def __init__(self, shapein=None, shapeout=None, **keywords):
+        ConstantOperator.__init__(self, 0, shapein=shapein, shapeout=shapeout,
+                                  **keywords)
         self.add_rule('.{Operator}', self._rule_zero_times)
         self.add_rule('{Operator}.', self._rule_times_zero)
 
